function Output = FindChannelProbabilities(N, OriginalErasureProbability)
 
% N - is the number of bits that are being processed in the polar transform
% p - is the erasure probability of the original channel W 
p              = OriginalErasureProbability;
NumberOfStages = log10(N)/log10(2);
if(rem(NumberOfStages, 1) ~= 0)
    error('N must be of the form 2^i, where i is an integer > 0.');
end
 
% The ChannelErasureProbabilities Matrix will hold the erasure probabilities, where the 
% last column holds the erasure probabilities of the original 2^N channels W, whereas
% column 1 will contain the erasure probabilities of the final polarized channels.
ChannelErasureProbabilities = zeros(N, NumberOfStages + 1);
ChannelErasureProbabilities(:, NumberOfStages + 1) = p*ones(N, 1);

% Iterate through each stage
for stage = NumberOfStages  :-1:1
    for channel = 1:N
        pPreviousChannel  = ChannelErasureProbabilities(channel, stage+1);
        MakeChannelBetter = mod( floor( (channel - 1)/(2^(stage-1)) ), 2);
        if(MakeChannelBetter == 1)
            ChannelErasureProbabilities(channel, stage) =  pPreviousChannel^2;
        else
            ChannelErasureProbabilities(channel, stage) =  2*pPreviousChannel - pPreviousChannel^2;
        end
    end
end
 
Output = ChannelErasureProbabilities;
return

FinalErasureProbabilities = ChannelErasureProbabilities(:, 1);
figure(1);
subplot(1,2,1);
hold off;
stem(1:N, FinalErasureProbabilities, 'k', 'Linewidth', 1); grid on; hold on;
plot(0:N, p*ones(1,N+1), 'k--');
axis([0 N 0 1]);
title(['Polarized Channel Erasure Probabilities N = ' num2str(N) ' and p = ' num2str(p)]);
xlabel('Channel Index');
ylabel('Probability');

subplot(1,2,2);
hold off;
stem(1:N, sort(FinalErasureProbabilities, 'descend'), 'k', 'Linewidth', 1); grid on; hold on;
plot(0:N, p*ones(1,N+1), 'k--');
axis([0 N 0 1]);
title(['Polarized Channel Erasure Probabilities N = ' num2str(N) ' and p = ' num2str(p)]);
xlabel('Reordered Channel Index');
ylabel('Probability');



 