clc;
clear variables;
close all

QamIndex      =  6;
TxSymbolValue =  (1/sqrt(42))*(-7 + 1j);
h             = -.1 + 0.5j;
NoisePower    = 0.001;
SignalPower   = (TxSymbolValue * conj(TxSymbolValue)) * h*conj(h);
SNR_dB        = 10*log10(SignalPower/NoisePower);
disp(['SNR_dB: ' num2str(SNR_dB)])

LlrBits = LlrDemapperIEEE(QamIndex, TxSymbolValue, h, NoisePower);
disp(['Llr Value: ' num2str(LlrBits)]);

% ------------------------------------------------------------------------------------------
%
% LlrDemap()
%
% ------------------------------------------------------------------------------------------
function LlrBits = LlrDemapperIEEE(QamIndex, SymbolValue, h, NoiseVar)
   LlrBits = zeros(1, QamIndex);
   switch(QamIndex)
      case 1       % BPSK
         y_I          = real(SymbolValue);
         LlrBits      = (4*h*conj(h)/NoiseVar)*y_I;                
      case 2       % QPSK
         y_I          = real(SymbolValue);
         y_Q          = imag(SymbolValue);
         LlrBits      = (4*h*conj(h)/(0.707*NoiseVar))*[y_I, y_Q];
      case 4       % 16QAM
         y_I          = real(SymbolValue);
         y_Q          = imag(SymbolValue);
         LlrBits(1,1) = GetLLR([+3, +1]/sqrt(10), [-3, -1]/sqrt(10), y_I, NoiseVar, h);
         LlrBits(1,2) = GetLLR([-1, +1]/sqrt(10), [-3,  3]/sqrt(10), y_I, NoiseVar, h);
         LlrBits(1,3) = GetLLR([+3, +1]/sqrt(10), [-3, -1]/sqrt(10), y_Q, NoiseVar, h);
         LlrBits(1,4) = GetLLR([-1, +1]/sqrt(10), [-3,  3]/sqrt(10), y_Q, NoiseVar, h);
      case 6       % 64QAM
         y_I          = real(SymbolValue);
         y_Q          = imag(SymbolValue);
         c            = sqrt(42);
         LlrBits(1,1) = GetLLR([ 7,  5, 3, 1]/c, [-7, -5, -3, -1]/c, y_I, NoiseVar, h);
         LlrBits(1,2) = GetLLR([-3, -1, 3, 1]/c, [-7, -5,  5,  7]/c, y_I, NoiseVar, h);
         LlrBits(1,3) = GetLLR([-5, -3, 3, 5]/c, [-7, -1,  1,  7]/c, y_I, NoiseVar, h);
         LlrBits(1,4) = GetLLR([ 7,  5, 3, 1]/c, [-7, -5, -3, -1]/c, y_Q, NoiseVar, h);
         LlrBits(1,5) = GetLLR([-3, -1, 3, 1]/c, [-7, -5,  5,  7]/c, y_Q, NoiseVar, h);
         LlrBits(1,6) = GetLLR([-5, -3, 3, 5]/c, [-7, -1,  1,  7]/c, y_Q, NoiseVar, h);
   end         
end % LlrDemapper()
 
 
function LLR_Value = GetLLR(Positions1, Positions0, y, NoiseVariance, h)
    Numerator        = 0;     % The denominator can remain at zero for very low noise variance.
    Denominator      = 1e-6;  % That's bad, so initialize it to a small value.
    NoiseVarEstimate = NoiseVariance / (h * conj(h)); % The channel modifies the noise variance.
    C1               = 1/sqrt(2*pi*NoiseVarEstimate);
    C2               = 2*NoiseVarEstimate;
    for i = 1:length(Positions1)  % Positions1 and Positions2 are row vectors
        a            = Positions1(1,i);
        Numerator    = Numerator   + C1*exp(-((y-a)^2)/C2); 
        a            = Positions0(1,i);
        Denominator  = Denominator + C1*exp(-((y-a)^2)/C2); 
    end
    LLR_Value = log(Numerator/Denominator); 
 end % GetLLR()
      