% File:   MinsumDecoder.m
% Author: Andreas Schwarzinger                    Date: 10/5/2021
% Notes:  This is the minsum encoder we will use for our successive cancellation decoder

clc;
clear variables;

CurrentNode  = 0;
CurrentStage = 1;
MovingLeft   = false;
IsMinSum     = false;

[NextStage, NextNode] = GetNextNode(CurrentNode   ...
                                  , CurrentStage  ...
                                  , MovingLeft    ...
                                  , IsMinSum);
 
disp(['Next Stage: ' num2str(NextStage) '   NextNode: ' num2str(NextNode)])                                
Done = 1;

PortIndices = GetPortIndices(CurrentNode, CurrentStage)


function OutputBelief = MinsumDecoder(Y1, Y2)
OutputBelief  = -sign(Y1)*sign(Y2)* min(abs(Y1), abs(Y2));
end


function OutputBelief = RepetitionDecoder(Y1, Y2, U1)

if(U1 <= 0); OutputBelief = Y2 + Y1;  % U1 can be belief or bit value
else;        OutputBelief = Y2 - Y1; end 
end


function [NextStage, NextNode] = GetNextNode(CurrentNode   ...
                                           , CurrentStage  ...
                                           , MovingLeft    ...
                                           , IsMinSum)
% If we are moving left, which is the decoding direction producing beliefs
if(MovingLeft == true)   
    NextStage   = CurrentStage - 1;      % As we are moving left
    NextNode    = CurrentNode * 2;       % If MinSum     decoder (Upper node)
    if(IsMinSum == false)                % If repetition decoder (Lower node)
        NextNode = NextNode + 1;
    end
% Else we are moving right, which is the re-encoding direction producing 0/1 bits.
else                      
    NextStage   = CurrentStage + 1;            % As we are moving right
    NextNode    = floor(CurrentNode / 2);
end
   
end


function PortIndices = GetPortIndices(CurrentNode, CurrentStage)

NumberOfPorts = 2^CurrentStage;
StartPort     = CurrentNode * NumberOfPorts;
PortIndices   = StartPort : StartPort + NumberOfPorts - 1;
end