function Output = PolarEncoder(InputVector)

if(nargin == 0)
    N = 16;
    InputVector = ones(1, N);
end

% N - is the number of bits that are being processed in the polar transform
N              = length(InputVector);
NumberOfStages = log10(N)/log10(2);
if(rem(NumberOfStages, 1) ~= 0)
    error('N must be of the form 2^i, where i is an integer > 0.');
end

% The matrix, M, shows the progression of the input bits through the Polar structure.
M       = zeros(N, NumberOfStages + 1);
M(:, 1) = InputVector(:);           % Load input vector into first column

% The polar encoding process
for stage = 1:NumberOfStages
    NumberInputsPerNode = 2^stage;                  %   2,   4,   8,   16, .... 
    NumberOfNodes       = N / NumberInputsPerNode;  % N/2, N/4, N/8, N/16, ....
    for node = 1:NumberOfNodes
        U1_StartIndex            = 2^(stage) * (node - 1) + 1;         
        U1_StopIndex             = U1_StartIndex + NumberInputsPerNode/2 - 1;
        U2_StartIndex            = U1_StopIndex + 1;
        U2_StopIndex             = U2_StartIndex + NumberInputsPerNode/2 - 1;
        U1_Indices               = U1_StartIndex : U1_StopIndex;
        U2_Indices               = U2_StartIndex : U2_StopIndex;
        U1                       = M(U1_Indices, stage);
        U2                       = M(U2_Indices, stage);
        X1                       = mod(U1 + U2, 2);
        X2                       = U2;
        M(U1_Indices, stage + 1) = X1;
        M(U2_Indices, stage + 1) = X2;
    end
end

Output = M(:, NumberOfStages + 1);
%display(M)



 