%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 5 - Section 2:  Differential Symbol Mapper
%% Copyright: Andreas Schwarzinger Nov 2021
%%
%% Author: Andreas Schwarzinger                          Date: Nov 2021

function OutputPositions = Differential_Symbol_Mapper(InputBits, Format)
% (modulation) Format: 0, 1, 2 --> pi_4_DQPSK, D8PSK, pi_8_D8PSK
 
persistent pi_4_DQPSK_LUT D8PSK_LUT pi_8_D8PSK_LUT
if(isempty(pi_4_DQPSK_LUT))
   Phase_Table1    = [pi/4; 3*pi/4; -pi/4; -3*pi/4];
   Phase_Table2    = [0;   pi;  3*pi; 2*pi; -pi; -2*pi;  4*pi; -3*pi]/4;  
   Phase_Table3    = [pi; 3*pi; -pi; -3*pi; 7*pi; 5*pi; -7*pi; -5*pi]/8;  
   
   pi_4_DQPSK_LUT  = cos(Phase_Table1) + 1j*sin(Phase_Table1);    
   D8PSK_LUT       = cos(Phase_Table2) + 1j*sin(Phase_Table2);    
   pi_8_D8PSK_LUT  = cos(Phase_Table3) + 1j*sin(Phase_Table3);  
end % Definition of Lookup Tables (LUT)

BitsPerSymbol = 0;
switch(Format)
    case 0; BitsPerSymbol = 2; 
    case 1; BitsPerSymbol = 3; 
    case 2; BitsPerSymbol = 3; 
end
    
NumberOfSymbols = fix( length(InputBits)/BitsPerSymbol );
OutputPositions = [1 zeros(1,NumberOfSymbols)];
 
for i = 2:NumberOfSymbols+1
    Start = 1 + (i - 2)*(BitsPerSymbol);
    Stop  = Start + BitsPerSymbol - 1;
    Bit_Group = InputBits(1,Start:Stop);
    switch(Format)
        case 0
            Code     = Bit_Group(1,1)*2 + Bit_Group(1,2); 
            Rotation = pi_4_DQPSK_LUT(Code+1, 1);
        case 1
            Code     = Bit_Group(1,1)*4 + Bit_Group(1,2)*2+Bit_Group(1,3); 
            Rotation = D8PSK_LUT(Code+1, 1);  
        case 2
            Code     = Bit_Group(1,1)*4 + Bit_Group(1,2)*2+Bit_Group(1,3);
            Rotation = pi_8_D8PSK_LUT(Code+1, 1);   
    end
    OutputPositions(1,i) = OutputPositions(1,i-1)*Rotation; 
end
