%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 5 - Section 2:  Calculating the Discrete Impulse Response of 
%%                         a Raised Cosine Pulse Shaping Filter
%% Copyright: Andreas Schwarzinger March 2015
%%
%% Author: Andreas Schwarzinger                          Date: March 2015

clc;             % clear the command line
clear variables; % clear memory
close all;       % close all previously opened figures

Fs    = 1e6;     % Symbol Rate
Alpha = 0.5;

%% Configuration 1   -- GetRaisedCosine(alpha, N, Oversampling)
OSR1 = 2;
OSR2 = 4;

[h1, t1] = GetRaisedCosine(Alpha,  9, OSR1, 0);
[h2, t2] = GetRaisedCosine(Alpha, 19, OSR2, 0);
[h5, t5] = GetRaisedCosine(Alpha, 39, 8, 0);  % I include this one only to display
                                              % the quasi continous raised cosine
                                              % shape that I show as dashed
                                              % lines in the plot
Hann  = hann(21);
h3 = h2 .* (Hann(2:20)').^.4;                 % This hanning window is raised to
                                              % to a power of 0.4
%figure(1);
%plot(t1, h1, 'k-o'); hold on; grid on;
%plot(t2, h2, 'b-o');

Len = length(h1);
f1 = 0:.005:2;             %% Valid Frequency Range
z = exp(1j*2*pi*f1);
 
H = zeros(1,length(f1));
for i = 1:Len
    Exponent = -(i-1);
    H = H + h1(1,i)*z.^Exponent;
end
     
MagResponse1    = abs(H);    %% Magnitude Part of Frequency Response
f1_Actual = Fs*f1*OSR1;

%[MagResponse1, PhaseResponse1, f1] = Show_NormalizedSpectrum(h1);
[MagResponse2, PhaseResponse2, f2] = Show_NormalizedSpectrum(h2);
[MagResponse3, PhaseResponse3, f3] = Show_NormalizedSpectrum(h3);
f2_Actual = Fs*f2*OSR2;

Log1 = 20*log10(MagResponse1);
Log2 = 20*log10(MagResponse2);
Log3 = 20*log10(MagResponse3);

%figure(2);
%plot(f1, Log1 - max(Log1), 'k'); grid on; hold on;
%plot(f2, Log2 - max(Log2), 'b');
%plot(f3, Log3 - max(Log3), 'r');


figure(3);
subplot(3,2,1);
stem(t1*1e-6, h1, 'k'); grid off; hold on; plot(t5*1e-6, h5, 'k--');
xlabel('time (sec)'); title('Impulse Response Configuration 1');
axis([-2.5e-6 2.5e-6 -0.2 1.2]);

subplot(3,2,2);
plot(f1_Actual, Log1 - max(Log1), 'k'); grid on;  title('Magnitude Response Configuration 1');
xlabel('frequency (Hz)'); ylabel('dB');
axis([0 Fs*max(f1)*OSR1 -60 0]);




subplot(3,2,3);
stem(t2*1e-6, h2, 'k'); grid off; hold on; plot(t5*1e-6, h5, 'k--');
xlabel('time (sec)'); title('Impulse Response Configuration 2');
axis([-2.5e-6 2.5e-6 -0.2 1.2]);
subplot(3,2,5);
stem(t2*1e-6, h3, 'k'); grid off; hold on; plot(t5*1e-6, h5, 'k--');
xlabel('time (sec)'); title('Impulse Response Configuration 3');
axis([-2.5e-6 2.5e-6 -0.2 1.2]);

subplot(3,2,4);
plot(f2_Actual, Log2 - max(Log2), 'k'); grid on; title('Magnitude Response Configuration 2');
xlabel('frequency (Hz)'); ylabel('dB');
axis([0 4e6 -60 0]);
subplot(3,2,6);
plot(f2_Actual, Log3 - max(Log3), 'k'); grid on; title('Magnitude Response Configuration 3');
axis([-0 2.5 -0.2 1.2]);
xlabel('frequency (Hz)'); ylabel('dB');
axis([0 4e6 -60 0]);