%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 5 - Section 2:  Symbol Mapper
%% Copyright: Andreas Schwarzinger Nov 2021
%%
%% Author: Andreas Schwarzinger                          Date: Nov 2021

function [OutputSymbols] = Symbol_Mapper(InputBits, ModulationFormat)
% Modulation format: 0, 1, 2, 3 --> BPSK, QPSK, 8PSK, 16QAM
persistent BPSK_LUT QPSK_LUT PSK8_LUT QAM16_LUT
if(isempty(BPSK_LUT))
   C         = 1/sqrt(2);
   BPSK_LUT  = 1/sqrt(2)* [-1 - 1j; 1 + 1j];      
   QPSK_LUT  = 1/sqrt(2)* [-1 - 1j; -1 + 1j;  1 - 1j;  1 + 1j];      
   PSK8_LUT  =            [-C + 1j*C; -1     ;    1j; C + 1j*C; ... 
                           -1j      ; -C - 1j*C; C - 1j*C;     1];   
   QAM16_LUT = 1/sqrt(10.66)*[-3 - 3j; -3 - 1j; -3 + 1j; -3 + 3j; ...  
                              -1 - 3j; -1 - 1j; -1 + 1j; -1 + 3j; ...   
                               1 - 3j;  1 - 1j;  1 + 1j;  1 + 3j; ...   
                               3 - 3j;  3 - 1j;  3 + 1j;  3 + 3j];
end
 
NumberOfSymbols = floor(length(InputBits)/(ModulationFormat + 1));
OutputSymbols   = zeros(1,NumberOfSymbols);
 
for i = 1:NumberOfSymbols
    Start = 1 + (i - 1)*(ModulationFormat + 1);
    Stop  = Start + ModulationFormat;
    Bit_Group = InputBits(1,Start:Stop);
    switch(ModulationFormat)
        case 0
            Code   = Bit_Group(1,1) + 1; Symbol = BPSK_LUT(Code, 1);
        case 1
            Code   = Bit_Group(1,2)*2 + Bit_Group(1,1) + 1;
            Symbol = QPSK_LUT(Code, 1);  
        case 2
            Code   = Bit_Group(1,3)*4 + Bit_Group(1,2)*2 ...
                   + Bit_Group(1,1) + 1;
            Symbol = PSK8_LUT(Code, 1);   
        case 3
            Code   = Bit_Group(1,4)*8 + Bit_Group(1,3)*4 ...
                   + Bit_Group(1,2)*2 + Bit_Group(1,1) + 1;
            Symbol = QAM16_LUT(Code, 1);
    end
    OutputSymbols(1,i) = Symbol;
end
