%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 5 - Section 2:  Pulse Shaping Using the (Root) Raised Cosing Pulse Shape
%% Copyright: Andreas Schwarzinger March 2015
%%
%% Author: Andreas Schwarzinger                          Date: March 2015

function [SampleStream, Zero_Stuffed, h] = TX_Waveform_Mapping( ...
       SymbolsStream, N, OSR, Alpha, Root, Hann_Flag, Hann_Index)
% N          - Number of taps in pulse shaping filter
% OSR        - Oversampling ratio --> Best if OSR >= 4
% Alpha      - Roll-off factor --> 0 (smallest BW)  to 1 (largest BW)
% Root       - Boolean value 0/1 - RC / RRC
% Hann_Flag  - Boolean value 0/1 - Skip / Use Hanning Window
% Hann_Index - The index modifies the Hanning Window
NumberOfSymbols                  =    length(SymbolsStream);

%% 1. The Zero Stuffing Process (there is nothing to it)
Zero_Stuffed                     = zeros(1, NumberOfSymbols*OSR);
for i = 1:NumberOfSymbols
    Zero_Stuffed(1, 1 + (i*OSR)) = SymbolsStream(1,i);
end
 
%% 2. Build Hanning Window
n          = 0:N-1;
HannWindow = (0.50-0.50*cos(2*pi*(n+1)/(N+1)) ).^Hann_Index ;
%% 3. Synthesize the proper RC or RRC Pulse Shape
[h, ~] = GetRaisedCosine(Alpha, N, OSR, Root);
if(Hann_Flag); h = h.*HannWindow; end 
 
%% 4. Pulse Shaping via RC filtering of Zero-stuffed Symbols
SampleStream = filter(h, 1, Zero_Stuffed);
