%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 5 - Section 3:  This template explores different version of Frequency shift keying.
%% Copyright: Andreas Schwarzinger March 2015
%%
%% Author: Andreas Schwarzinger                          Date: March 2015

clc;
clear variables;
close all;

Fs    = 2e8;        %% Sample Rate
Fo    = 6.25e6;     %% Carrier Frequency
OSR   = 100;        %% Oversampling Ratio
a0    = -3*1.25e6;  %% Symbol a0 = -3.75MHz
a1    = -1*1.25e6;  %% Symbol a1 = -1.25MHz
a2    =  1*1.25e6;  %% Symbol a2 =  1.25MHz
a3    =  3*1.25e6;  %% Symbol a3 =  3.75MHz

fd   = [a3*(ones(1,OSR)) a0*(ones(1,OSR)) a1*(ones(1,OSR)) ...
        a2*(ones(1,OSR))];

Theta           = zeros(1,4*OSR);                %% Theta(fd,t)
for n = 2:(4*OSR)
    Theta(1,n)  = Theta(1,n-1) + 2*pi*fd(n)/Fs;  % Accumulation
end

RF_Signal  = cos(2*pi*(0:(4*OSR - 1))*Fo./Fs + Theta);

RF_Signal2 = cos(Theta).*cos(2*pi*(0:(4*OSR - 1))*Fo./Fs) -   sin(Theta).*sin(2*pi*(0:(4*OSR - 1))*Fo./Fs);

figure(1);
subplot(2,1,1);
plot((1:4*OSR)/Fs, fd, 'k', 'Linewidth',2); grid on; title('fd(t)'); 
grid on;  axis([0 2e-6 -50e5 50e5]); 
xlabel('time (sec)'); ylabel('frequency (Hz)');
set(gca, 'XTick', [0 0.5e-6 1e-6 1.5e-6]); set(gca, 'YTick', [a0 a1 a2 a3]);
subplot(2,1,2);
plot((1:4*OSR)/Fs, RF_Signal, 'k'); grid on; title('RF(t)'); hold on;
plot((1:4*OSR)/Fs, RF_Signal2, 'k');  
xlabel('time (sec)');
set(gca, 'XTick', [0 0.5e-6 1e-6 1.5e-6]);
set(gca, 'YTick', [-1 0 1]);


