%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 5 - Section 6    This test bench will validate the performance of our Successive Cancellation Polar decoder.
%% Copyright: Andreas Schwarzinger Oct 2021
%%
%% Author: Andreas Schwarzinger                              Oct 2021

 
clc;
% Configure the polar encoder
N                = 2048;               % Number of total encoded bits
K                = 680;                % Number of message bits
NumberFrozenBits = N - K;              % Number of frozen positions set to 0
EncoderRate      = K/N;                % The encoder rate
MessageBits      = randi([0 1], K, 1);

% Let's determine the erasure probability of each channel
p                          = 0.2;   % The erasure probabilities might change a little with p.
ErasureProbabilitiesMatrix = FindChannelProbabilities(N, p );
ErasureProbabilities       = ErasureProbabilitiesMatrix(:,1);
[SortedProbabilities, SortedIndices] = sort(ErasureProbabilities, 'descend');

IndexOfFrozenBits  = SortedIndices(1:NumberFrozenBits, 1);
IndexOfMessageBits = SortedIndices(NumberFrozenBits+1: end, 1);

% Produce the bits to be transmitted through the polar encoder
TxBits = zeros(1, N);
TxBits(1, IndexOfMessageBits) = MessageBits;
EncodedBits        = PolarEncoder(TxBits);
EncodedBipolarBits = 2*EncodedBits - 1;    % Map from 0/1 to -1/1

% The Encoded Bipolar Bits are now disturbed by noise to produce received beliefs
SNR_dB      = 1;
SNR_Linear  = 10^(SNR_dB/10);
SignalPower = 1;   % = MeanSquare(EncodedBipolarBits)
NoisePower  = SignalPower/SNR_Linear;
AWG_Noise   = sqrt(NoisePower) * randn(N, 1);
Y           = EncodedBipolarBits + AWG_Noise;

% Run the Successive Cancellation Decoder
IndexOfFrozenBits0Based = IndexOfFrozenBits - 1;
DecodedBits             = PolarDecoderBook(Y, IndexOfFrozenBits0Based);
DecodedMessageBits      = DecodedBits(IndexOfMessageBits, 1);

% Compute number of errors and bit error rate
NumberOfErrors = sum(mod(MessageBits + DecodedMessageBits, 2));
BER            = NumberOfErrors / K;
disp(['Encoder Rate:           ' num2str(EncoderRate)]);
disp(['Number of Message Bits: ' num2str(K)]);
disp(['Number of Errors:       ' num2str(NumberOfErrors)]);
disp(['BER:                    ' num2str(BER)]);