%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 7 - Section 1: Frequency Response Estimation Example and Code
%% Copyright: Andreas Schwarzinger May 2020
%%
%% Author: Andreas Schwarzinger  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;
clear variables;

%% 1. Determine Channel Model with Four Paths

              %    Cp      Delay(sec)   Doppler
ChannelModel    = [0.5,       0e-6,      20;  ... % Path 0 (The earliest path)
                    1j,     0.5e-6,    -100;  ... % Path 1 (The strongest path)
                  0.5j,     2.5e-6,      50;  ... % Path 2
                   0.2,     4.5e-6,     150];     % Path 3 (The Latest path)

%% 2. Compute the frequency response
f            = -1.5e6:15000:1.5e6;                     
Time         = 0.001;
FreqResponse = ComputeFrequencyResponse(ChannelModel, f, Time);            
            
figure(1);
subplot(2,2,1);
plot(f, abs(FreqResponse), 'k'); grid on;
title('Magnitude Response');
xlabel('frequency (Hz)');
axis([min(f), max(f), 0, 2.0]);
subplot(2,2,3);
plot(f, angle(FreqResponse), 'k'); grid on;
title('Phase Response');
xlabel('frequency (Hz)');
subplot(2,2,[2, 4]);
plot(f, real(FreqResponse), 'k'); grid on; hold on;
plot(f, imag(FreqResponse), 'k--'); hold off;
title('Real and Imag Portion of Frequency Response');
xlabel('frequency (Hz)');
legend('Real', 'Imag')
%subplot(2,2,4);
%plot(f, imag(FreqResponse), 'k'); grid on;
%title('Imag Portion of Frequency Response');
%xlabel('frequency (Hz)');
            
            
%% 3. Compute the frequency model according to equation 7.2            
function FreqResponse = ComputeFrequencyResponse(ChannelModel  ... % Our Multipath Channel Model
                                               , f             ... % The frequencies and the time at
                                               , Time)             % which to evaluate the Frequency Response
FreqResponse = zeros(size(f));
NumPaths     = size(ChannelModel, 1);      % Get number of rows of the matrix

for p = 1:NumPaths
   Cp      = ChannelModel(p, 1);           % Complex scalar
   Tau_p   = ChannelModel(p, 2);           % The delay 
   f_p     = ChannelModel(p, 3);           % The Doppler Shift
   FreqResponse = FreqResponse + ...
                  Cp * exp(-1j*2*pi*Tau_p*f) * exp(1j*2*pi*f_p*Time);
end

end