%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 7 - Section 5: This function implements layer mapping for broadcast information in LTE
%% Copyright: Andreas Schwarzinger Nov 2021
%%
%% Author: Andreas Schwarzinger                              Nov 2021

function [Layer0, Layer1, Layer2, Layer3] = LayerMapping(ModulatedSymbols_Tx)
Layer1 = []; Layer2 = []; Layer3 = [];
 
if(NumLtePorts == 1)               % Single Antenna port
   Layer0  = ModulatedSymbols_Tx;  % There are 960 or 864 modulated symbols in this layer
elseif(NumLtePorts == 2)           % Two Antenna ports
   NumSymbolsPerLayer = NumModulatedSymbols/2;
   ModSymb            = reshape(ModulatedSymbols_Tx, [2 NumSymbolsPerLayer]); 
   Layer0             = ModSymb(1, :);        % Either 480 or 432 bits in each layer
   Layer1             = ModSymb(2, :);          
else                                % Four Antenna ports
   if(mod(NumModulatedSymbols,4) == 0)
       NumSymbolsPerLayer = NumModulatedSymbols/4;
       ModSymb            = reshape(ModulatedSymbols_Tx, [4 NumSymbolsPerLayer]);
   else
       NumSymbolsPerLayer           = (NumModulatedSymbols+2)/4;
       ModulatedSymbols_Tx_Extended = [ModulatedSymbols_Tx 0 0]; % Appending 2 zeros
       ModSymb = reshape(ModulatedSymbols_Tx_Extended, [4 NumSymbolsPerLayer]);
   end
   Layer0  = ModSymb(1, :);          % Either 240 or 216 bits in each layer
   Layer1  = ModSymb(2, :);           
   Layer2  = ModSymb(3, :);         
   Layer3  = ModSymb(4, :);          
end


