%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 7 - Section 5: This function implements layer mapping for the physical broadcast channel.
%% Copyright: Andreas Schwarzinger Nov 2021
%%
%% Author: Andreas Schwarzinger                              Nov 2021

function [PrecodedP0, PrecodedP1, PrecodedP2, PrecodedP3] = PBCH_Precoding(Layer0, Layer1, Layer2, Layer3)
PrecodedP0 = [];
PrecodedP1 = [];
PrecodedP2 = [];
PrecodedP3 = [];
if(NumLtePorts == 1)
        PrecodedP0 = Layer0;
elseif(NumLtePorts == 2)
    % This is the SFBC precoding step for the 2 Tx antenna scenario  
    for i = 0:NumSymbolsPerLayer                                
        PrecodedP0(1, 2*i + 0 + 1, 1) = (1/sqrt(2))*Layer0(1, i+1);
        PrecodedP0(1, 2*i + 1 + 1, 1) = (1/sqrt(2))*Layer1(1, i+1);    
        PrecodedP1(1, 2*i + 0 + 1, 1) = (1/sqrt(2))*conj(-Layer1(1, i+1));
        PrecodedP1(1, 2*i + 1 + 1, 1) = (1/sqrt(2))*conj( Layer0(1, i+1));
    end
elseif(NumLtePort2 == 4)
    % This is the staggered SFBC precoding step for the 4 TX Antenna scenario
    for i = 0:NumSymbolsPerLayer     
        PrecodedP0(1, 4*i + 0 + 1, 1) = (1/sqrt(2))*Layer0(1, i+1);
        PrecodedP0(1, 4*i + 1 + 1, 1) = (1/sqrt(2))*Layer1(1, i+1);    
        PrecodedP2(1, 4*i + 0 + 1, 1) = (1/sqrt(2))*conj(-Layer1(1, i+1));
        PrecodedP2(1, 4*i + 1 + 1, 1) = (1/sqrt(2))*conj( Layer0(1, i+1));
        
        PrecodedP1(1, 4*i + 2 + 1, 1) = (1/sqrt(2))*Layer2(1, i+1);
        PrecodedP1(1, 4*i + 3 + 1, 1) = (1/sqrt(2))*Layer3(1, i+1);    
        PrecodedP3(1, 4*i + 2 + 1, 1) = (1/sqrt(2))*conj(-Layer2(1, i+1));
        PrecodedP3(1, 4*i + 3 + 1, 1) = (1/sqrt(2))*conj( Layer3(1, i+1));
    end
end

