%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 8 - Section 2: A simple example of SFBC Encoding and Decoding as used in LTE
%% Copyright: Andreas Schwarzinger Oct 2021

clc;
clear variables;

%% An Example of SFBC as used in LTE
%  Let's define the following input symbols
x0 =  1.0 + 1.0j;     % The source symbol x[n]
x1 = -0.5 - 2.0j;     % The source symbol x[n+1]
 
% Defining the Channel
% hsymbol,Ant - so h00 and h10 come from the same antenna but different subcarriers
h00 =  1 + 0.2j;    % Channel coefficient (symbol0, Antenna0)
h10 =  1 - 1j;      % Channel coefficient (symbol1, Antenna0)
h01 =  2 + 1j;      % Channel coefficient (symbol0, Antenna1)
h11 = -3 - 1.2j;    % Channel coefficient (symbol1, Antenna1)
% The observations are defined as follows
y0 = h00*x0       - h01*conj(x1);      % The observation y[0]
y1 = h11*conj(x0) + h10*x1;            % The observation y[1]
 

% The observation vector, Y, is defined as follows
Y  = [y0; conj(y1)];
 
% The quasi channel matrix
H  = [     h00       -h01; ...
      conj(h11)  conj(h10)];
 
% Method 1: Finding the Estimates of x0 and x1 using the inv(H) function
X_Estimate1  = inv(H)*Y;
x0_Estimate1 = X_Estimate1(1,1);                % The estimate of x[0]
x1_Estimate1 = conj(X_Estimate1(2,1));          % The estimate of x[1]
 
% Method 2: Or using the explicit formula for the 2x2 inverse
c            =  1/(h00*conj(h10) + conj(h11)*h01);
X_Estimate2  =  c * [conj(h10) h01; -conj(h11) h00] * Y;
x0_Estimate2 = X_Estimate2(1,1);               % The estimate of x[0]
x1_Estimate2 = conj(X_Estimate2(2,1));         % The estimate of x[1]
