%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 8 - Section 2: A simple example of STBC Encoding and Decoding
%% Copyright: Andreas Schwarzinger Oct 2021

clc;
clear variables;

%% The Space time block code example
x0 =  1.0 + 1.0j;        % Used as x[n]
x1 = -0.5 - 2.0j;        % Used as x[n+1]

% Defining the Channel
h1 =  1 + 0.2j;    % Channel coefficient (from TX Antenna1)
h2 =  1 - 1j;      % Channel coefficient (from TX Antenna2)

% The observations are defined via the following encoding scheme
y0 =  h1*x0       + h2*x1;             % Represents the observation y[n]
y1 = -h1*conj(x1) + h2*conj(x0);       % Represents the observation y[n+1]

% The estimates x_0 and x_1 nicely reveal the original transmit symbols
c   = 1/(conj(h1)*h1 + conj(h2)*h2);
x_0 = c*(conj(h1)*y0 + h2*conj(y1));   % Estimate of x[n]
x_1 = c*(conj(h2)*y0 - h1*conj(y1));   % Estimate of x[n+1]

disp(x_0)
disp(x_1)