%% Text: Digital Signal Processing in Modern Communication Systems (Edition 2)
%% Chapter 8 - Section 3: Example 8.2 
%% Copyright: Andreas Schwarzinger May 2021
%%
%% Author: Andreas Schwarzinger                              May 2021

clc;
clear variables;
close all;

% QPSK source, x1[n], and BPSK Source, x2[n]
NumSymbols = 200;
x1   = 0.7071*((randi([0 1],1,NumSymbols)*2 - 1) + 1j*(randi([0 1],1,NumSymbols)*2-1));
x2   = randi([0 1], 1, NumSymbols)*2 - 1;
X    = [x1; ...    % The x1[n] QPSK symbol stream
        x2];       % The x2[n] BPSK symbol stream

    
% Design the Channel Matrix. 
ChannelSelection = 1;
switch(ChannelSelection)
    case 1; H  = [-1.0,  0.1 ; 0.1, -1.0 ];
    case 2; H  = [-1.0, -0.15; 1.0, -0.15];
    case 3; H  = [-0.8,  0.6 ; 0.6, -0.8 ];
    otherwise; error('Unsupported Channel Selection');
end

% Pass the information through the Channel Matrix add noise to form the 
NoisePower = 0.01;                                              %% observation Y
VN         = sqrt(NoisePower)*0.7071*(randn(2, NumSymbols) + 1j*randn(2, NumSymbols));
Y          = H*X + VN;    % The observation column vector


% Compute the Estimator Output
EstimatorMatrix = inv(H);  % The Estimator is simply the inverse of the Channel Matrix
X_Estimate      = EstimatorMatrix * Y;

QPSK_Estimate   = X_Estimate(1, :);
BPSK_Estimate   = X_Estimate(2, :);

figure(1);
subplot(1,2,1);
plot(real(QPSK_Estimate), imag(QPSK_Estimate), 'ko'); grid on;
title('The Estimated QPSK Data Stream');
xlabel('Real');
ylabel('Imag');
axis([-1.2 1.2 -1.2 1.2]);

subplot(1,2,2);
plot(real(BPSK_Estimate), imag(BPSK_Estimate), 'ko'); grid on;
title('The Estimated BPSK Data Stream');
xlabel('Real');
ylabel('Imag');
axis([-1.2 1.2 -1.2 1.2]);